@@ -1,6 +1,10 @@
 /* Built-in files */
 
-/* while IFS= read -r x; do x=${x//\\/\\\\}; print -r -- $'\t\t'\""${x//\"/\\\"}\n"\"; done */
+/*
+	<jupprc grep -v $'^[\t ]' | while IFS= read -r x; do
+		x=${x//\\/\\\\}; print -r -- $'\t\t'\""${x//\"/\\\"}\n"\";
+	done | perl -pi -e 's/[^\ca-~]/sprintf "\\x%02X", unpack("U", $&)/eg'
+ */
 
 #include "types.h"
 
@@ -16,6 +20,7 @@ const unsigned char * const builtins[]=
 		"-nobackups\n"
 		"-noxon\n"
 		"-notite\n"
+		"-pastetite\n"
 		"-pg 2\n"
 		"-lmsg \\i%k%T%*\\b%n\\b%R\n"
 		"-rmsg  R%r<%l C%c\\u%o|%O\\i\\b%a|%A\\b\\i\\u %u\n"
@@ -24,7 +29,7 @@ const unsigned char * const builtins[]=
 		"-french\n"
 		"-indentc 9\n"
 		"-istep 1\n"
-		"-guess_indent\n"
+		"--guess_indent\n"
 		"--autoindent\n"
 		"-purify\n"
 		"-highlight\n"
@@ -39,7 +44,6 @@ const unsigned char * const builtins[]=
 		"\n"
 		"*.py\n"
 		"-encoding utf8\n"
-		"--guess_indent\n"
 		"-tab 4\n"
 		"-indentc 32\n"
 		"-istep 4\n"
@@ -48,7 +52,6 @@ const unsigned char * const builtins[]=
 		"*\n"
 		"+#!\\+\\[	 ]\\+\\[a-z/]/python\n"
 		"-encoding utf8\n"
-		"--guess_indent\n"
 		"-tab 4\n"
 		"-indentc 32\n"
 		"-istep 4\n"
@@ -57,7 +60,6 @@ const unsigned char * const builtins[]=
 		"*\n"
 		"+#!\\+\\[	 ]\\+\\[a-z/]/env\\+\\[	 ]python\n"
 		"-encoding utf8\n"
-		"--guess_indent\n"
 		"-tab 4\n"
 		"-indentc 32\n"
 		"-istep 4\n"
@@ -72,36 +74,41 @@ const unsigned char * const builtins[]=
 		"\\i \\i^F next word     ^QD end of line  ^K/ pipe ^QY >line ^O  options  \\u\\bFILE\\b\\u       \\i \\i\n"
 		"\\i \\i\\u\\bSEARCH\\b\\u           ^QR top of file  ^KB begin          \\u\\bBUFFER\\b\\u       ^KE new    \\i \\i\n"
 		"\\i \\i^QF find first   ^QC end of file  ^KK end  ^K] space ^U undo      ^KR import \\i \\i\n"
-		"\\i \\i^L  find next    ^QI to line No.  ^KD reformat       ^^ redo      ^KW export \\i \\i\n"
+		"\\i \\i^L  find next    ^QO byte offset  ^KD reformat       ^^ redo      ^KW export \\i \\i\n"
 		"}\n"
 		"\n"
 		"{Windows\n"
 		"\\i   Help Screen    turn off with ^J     prev. screen ^[,    next screen ^[.     \\i\n"
-		"\\i \\i^KO Split the window in half         ^KE Load file into window               \\i \\i\n"
+		"\\i \\i^KO Split the window in half         ^KE Load file into new window           \\i \\i\n"
 		"\\i \\i^KG Make current window bigger       ^KT Make current window smaller         \\i \\i\n"
 		"\\i \\i^KN Go to the window below           ^KP Go to the window above              \\i \\i\n"
 		"\\i \\i^KQ Eliminate the current window     ^KI Show all windows / Show one window  \\i \\i\n"
+		"\\i \\i^K; Do ctags search into new window  ^K- Edit scratch buffer in new window   \\i \\i\n"
+		"\\i \\i Note: some commands (^KE ^K; ^K-) hide the current window. Use ^KI/^KN then.\\i \\i\n"
 		"\\i \\i\\u\\bSpecial help for XON/XOFF aware terminals\\b\\u                                    \\i \\i\n"
 		"\\i \\i You can use \\b^[q\\b and \\b^[s\\b instead of \\b^Q\\b and \\b^S\\b to initiate a command.         \\i \\i\n"
 		"}\n"
 		"\n"
 		"{Advanced\n"
 		"\\i   Help Screen    turn off with ^J     prev. screen ^[,    next screen ^[.     \\i\n"
-		"\\i \\i\\u\\bMACROS\\b\\u      \\u\\bMISC\\b\\u          \\u\\bSCROLL\\b\\u      \\u\\bSHELL\\b\\u       \\u\\bGOTO\\b\\u        \\u\\bI-SEARCH\\b\\u       \\i \\i\n"
-		"\\i \\i^[( Record  ^Q? status    ^Q. Right   ^K' Window  ^QB to ^KB  ^[R Backwards  \\i \\i\n"
-		"\\i \\i^[) Stop    ^QQ repeat    ^Q, Left    ^[! Command ^QK to ^KK  ^[S Forwards   \\i \\i\n"
-		"\\i \\i^[ 0-9 Play ^QM Math      \\u\\bCharSEARCH\\b\\u  \\u\\bQUOTE\\b\\u       \\u\\bDELETE\\b\\u      \\u\\bBOOKMARKS\\b\\u      \\i \\i\n"
-		"\\i \\i^[? Query   ^KA CenterLn. ^QH forwrd   ` Ctrl-    ^[Y yank    ^K 0-9 Set     \\i \\i\n"
-		"\\i \\i^[D Dump    ^[H Message   ^QG backwd  ^P Meta-    ^[O word<   ^Q 0-9 Goto    \\i \\i\n"
+		"\\i \\i\\u\\bMACROS\\b\\u       \\u\\bMISC\\b\\u         \\u\\bSCROLL\\b\\u      \\u\\bSHELL\\b\\u       \\u\\bGOTO\\b\\u        \\u\\bI-SEARCH\\b\\u       \\i \\i\n"
+		"\\i \\i^[( Record   ^Q? status   ^Q. Right   ^K' Window  ^QB to ^KB  ^[R Backwards  \\i \\i\n"
+		"\\i \\i^[) Stop     ^QQ repeat   ^Q, Left    ^[! Command ^QK to ^KK  ^[S Forwards   \\i \\i\n"
+		"\\i \\i^[? Query    ^QM Math     \\u\\bCharSEARCH\\b\\u  \\u\\bQUOTE\\b\\u       \\u\\bDELETE\\b\\u      \\u\\bBOOKMARKS\\b\\u      \\i \\i\n"
+		"\\i \\i^[D Dump     ^[H Message  ^QH forwrd   ` Ctrl-    ^[Y yank    ^K 0-9 Set     \\i \\i\n"
+		"\\i \\i^[ 0-9 Play  ^N  Play #0  ^QG backwd  ^P Meta-    ^[O word<   ^Q 0-9 Goto    \\i \\i\n"
+		"\\i \\i \\u\\bIn math mode\\b\\u, use 0xCAFE for hex. All ops are floating point internally.    \\i \\i\n"
+		"\\i \\i \\u\\bPredefined variables:\\b\\u byte col height line lines top width                  \\i \\i\n"
 		"}\n"
 		"\n"
 		"{Programs\n"
 		"\\i   Help Screen    turn off with ^J     prev. screen ^[,    next screen ^[.     \\i\n"
-		"\\i \\i\\u\\bGOTO\\b\\u                  \\u\\bINDENT\\b\\u         \\u\\bCOMPILING\\b\\u                   \\u\\bMACROS\\b\\u      \\i \\i\n"
-		"\\i \\i^Q[ Matching brace    ^K. more       ^[C Compile & parse errors  ^N play #0  \\i \\i\n"
-		"\\i \\i^QP Previous place    ^K, less       ^[E Parse errors            \\u\\bSLIDING\\b\\u     \\i \\i\n"
-		"\\i \\i^K= Next place        \\u\\bSPECIAL\\b\\u        ^[M Goto next error         ^W up       \\i \\i\n"
-		"\\i \\i^K; Split+Tag search  ^[- jexecmd    ^[N Goto previous error     ^Z down     \\i \\i\n"
+		"\\i \\i\\u\\bGOTO\\b\\u                  \\u\\bCOMPILING\\b\\u                    \\u\\bINDENT\\b\\u       \\u\\bSLIDING\\b\\u      \\i \\i\n"
+		"\\i \\i^Q- to column number  ^[C Compile & parse errors   ^K. more     ^W up        \\i \\i\n"
+		"\\i \\i^QI to line number    ^[E Parse errors             ^K, less     ^Z down      \\i \\i\n"
+		"\\i \\i^QP previous place    \\u\\bGOTO AFTER COMPILING\\b\\u         ^KA center   \\u\\bINSERT MATH\\b\\u  \\i \\i\n"
+		"\\i \\i^K= next place        ^[N previous error    \\u\\bSPECIAL\\b\\u             ^[# equation \\i \\i\n"
+		"\\i \\i^Q[ matching brace    ^[M next error        ^[- exec. juppcmd   ^[= result   \\i \\i\n"
 		"}\n"
 		"\n"
 		"{Search\n"
@@ -135,7 +142,7 @@ const unsigned char * const builtins[]=
 		"\\i \\i go to \\uhttp://sf.net/projects/joe-editor/\\u for upstream bug reports. JUPP 2.8 \\i \\i\n"
 		"\\i \\i for DOS compiled by A. Totlis, packed with LHarc 2.13; JUPP 3.x for UNIX\\d(R)\\d \\i \\i\n"
 		"\\i \\i at \\uhttp://mirbsd.de/jupp\\u and by \\bThorsten \"\\dmirabilos\\d\" Glaser <\\utg@mirbsd.org\\u>\\b \\i \\i\n"
-		"\\i \\i @(#) blt_in 2011-10-04; 3.1; autoCR-LF; UTF-8 via locale; per-file encoding \\i \\i\n"
+		"\\i \\i @(#) blt_in 2013-01-05; 3.1; autoCR-LF; UTF-8 via locale; per-file encoding \\i \\i\n"
 		"}\n"
 		"\n"
 		"{CharTable\n"
@@ -144,16 +151,21 @@ const unsigned char * const builtins[]=
 		"\\i \\i         |                                              |                    \\i \\i\n"
 		"\\i \\i   0  00 | \\u@ABC\\u \\uDEFG\\u  \\uHIJK\\u \\uLMNO\\u    \\i\\u@ABC\\u\\i \\i\\uDEFG\\u\\i  \\i\\uHIJK\\u\\i \\i\\uLMNO\\u\\i | 80  128            \\i \\i\n"
 		"\\i \\i  16  10 | \\uPQRS\\u \\uTUVW\\u  \\uXYZ[\\u \\u\\\\]^_\\u    \\i\\uPQRS\\u\\i \\i\\uTUVW\\u\\i  \\i\\uXYZ[\\u\\i \\i\\u\\\\]^_\\u\\i | 90  144            \\i \\i\n"
-		"\\i \\i  32  20 |  !\"# $%&'  ()*+ ,-./         | A0  160            \\i \\i\n"
-		"\\i \\i  48  30 | 0123 4567  89:; <=>?         | B0  176            \\i \\i\n"
-		"\\i \\i  64  40 | @ABC DEFG  HIJK LMNO         | C0  192            \\i \\i\n"
-		"\\i \\i  80  50 | PQRS TUVW  XYZ[ \\\\]^_         | D0  208            \\i \\i\n"
-		"\\i \\i  96  60 | `abc defg  hijk lmno         | E0  224            \\i \\i\n"
-		"\\i \\i 112  70 | pqrs tuvw  xyz{ |}~         | F0  240            \\i \\i\n"
+		"\\i \\i  32  20 |  !\"# $%&'  ()*+ ,-./    \xA0\xA1\xA2\xA3 \xA4\xA5\xA6\xA7  \xA8\xA9\xAA\xAB \xAC\xAD\xAE\xAF | A0  160            \\i \\i\n"
+		"\\i \\i  48  30 | 0123 4567  89:; <=>?    \xB0\xB1\xB2\xB3 \xB4\xB5\xB6\xB7  \xB8\xB9\xBA\xBB \xBC\xBD\xBE\xBF | B0  176            \\i \\i\n"
+		"\\i \\i  64  40 | @ABC DEFG  HIJK LMNO    \xC0\xC1\xC2\xC3 \xC4\xC5\xC6\xC7  \xC8\xC9\xCA\xCB \xCC\xCD\xCE\xCF | C0  192            \\i \\i\n"
+		"\\i \\i  80  50 | PQRS TUVW  XYZ[ \\\\]^_    \xD0\xD1\xD2\xD3 \xD4\xD5\xD6\xD7  \xD8\xD9\xDA\xDB \xDC\xDD\xDE\xDF | D0  208            \\i \\i\n"
+		"\\i \\i  96  60 | `abc defg  hijk lmno    \xE0\xE1\xE2\xE3 \xE4\xE5\xE6\xE7  \xE8\xE9\xEA\xEB \xEC\xED\xEE\xEF | E0  224            \\i \\i\n"
+		"\\i \\i 112  70 | pqrs tuvw  xyz{ |}~\x7F    \xF0\xF1\xF2\xF3 \xF4\xF5\xF6\xF7  \xF8\xF9\xFA\xFB \xFC\xFD\xFE\xFF | F0  240            \\i \\i\n"
+		"}\n"
+		"\n"
+		"{Paste\n"
+		"\\i                                                                               \\i\n"
+		"\\i \\i \\u\\bPaste Mode\\b\\u     turn off with \\b^D\\b or \\b^[[201~\\b                                  \\i \\i\n"
 		"}\n"
 		"\n"
 		":windows\n"
-		"type		^@ TO \n"
+		"type		^@ TO \xFF\n"
 		"abort		^K Q\n"
 		"abort		^K ^Q\n"
 		"abort		^K q\n"
@@ -172,6 +184,8 @@ const unsigned char * const builtins[]=
 		"math		^Q ^M\n"
 		"math		^Q m\n"
 		"math		^[ q m\n"
+		"mathins		^[ #\n"
+		"mathres		^[ =\n"
 		"msg		^[ H\n"
 		"msg		^[ h\n"
 		"nextw		^K N\n"
@@ -195,23 +209,37 @@ const unsigned char * const builtins[]=
 		"shell		^K z\n"
 		"stop		^[ )\n"
 		"\n"
+		":Paste\n"
+		"type					^@ TO \xFF\n"
+		"rtn					^M\n"
+		"msg,\"Entered bracketed paste mode\",rtn	^[ [ 2 0 0 ~\n"
+		"helpcard,rtn,keymap,\"main\",rtn,msg,rtn	^[ [ 2 0 1 ~\n"
+		"helpcard,rtn,keymap,\"main\",rtn		^D\n"
+		"\n"
 		":main\n"
 		":inherit windows\n"
 		"bof,qrepl,\"\\\\[\",quote,\"i\",quote,\"k\",quote,\"l\",quote,\"m ]\\\\+\\\\[\",quote,\"i\",quote,\"k\",quote,\"l\",quote,\"m ]\\\\$\",rtn,rtn,rtn,\"r\",eof	^K ]\n"
 		"edit,rtn,filt,query,parserr	^[ C\n"
 		"edit,rtn,filt,query,parserr	^[ c\n"
+		"helpcard,\"Paste\",rtn,keymap,\"Paste\",rtn	^[ P\n"
+		"helpcard,\"Paste\",rtn,keymap,\"Paste\",rtn	^[ p\n"
+		"helpcard,\"Paste\",rtn,keymap,\"Paste\",rtn	^[ [ 2 0 0 ~\n"
 		"begin_marking,uparw,toggle_marking	^[ [ 1 ; 2 A\n"
 		"begin_marking,dnarw,toggle_marking	^[ [ 1 ; 2 B\n"
 		"begin_marking,rtarw,toggle_marking	^[ [ 1 ; 2 C\n"
 		"begin_marking,ltarw,toggle_marking	^[ [ 1 ; 2 D\n"
+		"begin_marking,bol,toggle_marking	^[ [ 1 ; 2 H\n"
+		"begin_marking,eol,toggle_marking	^[ [ 1 ; 2 F\n"
+		"begin_marking,bof,toggle_marking	^[ [ 1 ; 6 H\n"
+		"begin_marking,eof,toggle_marking	^[ [ 1 ; 6 F\n"
 		"backs		^?\n"
 		"backs		^H\n"
 		"backw		^[ o\n"
 		"bknd		^K '\n"
-		"bkwdc		^Q G ^@ TO \n"
-		"bkwdc		^Q ^G ^@ TO \n"
-		"bkwdc		^Q g ^@ TO \n"
-		"bkwdc		^[ q g ^@ TO \n"
+		"bkwdc		^Q G ^@ TO \xFF\n"
+		"bkwdc		^Q ^G ^@ TO \xFF\n"
+		"bkwdc		^Q g ^@ TO \xFF\n"
+		"bkwdc		^[ q g ^@ TO \xFF\n"
 		"blkcpy		^K C\n"
 		"blkcpy		^K ^C\n"
 		"blkcpy		^K c\n"
@@ -241,9 +269,14 @@ const unsigned char * const builtins[]=
 		"bos		^Q ^X\n"
 		"bos		^Q x\n"
 		"bos		^[ q x\n"
+		"byte		^Q O\n"
+		"byte		^Q ^O\n"
+		"byte		^Q o\n"
+		"byte		^[ q o\n"
 		"center		^K A\n"
 		"center		^K ^A\n"
 		"center		^K a\n"
+		"col		^Q -\n"
 		"crawll		^Q ,\n"
 		"crawll		^[ q ,\n"
 		"crawlr		^Q .\n"
@@ -299,10 +332,10 @@ const unsigned char * const builtins[]=
 		"fnext		^L\n"
 		"fnext		^[ [ 1 3 ~\n"
 		"format		^B\n"
-		"fwrdc		^Q H ^@ TO \n"
-		"fwrdc		^Q ^H ^@ TO \n"
-		"fwrdc		^Q h ^@ TO \n"
-		"fwrdc		^[ q h ^@ TO \n"
+		"fwrdc		^Q H ^@ TO \xFF\n"
+		"fwrdc		^Q ^H ^@ TO \xFF\n"
+		"fwrdc		^Q h ^@ TO \xFF\n"
+		"fwrdc		^[ q h ^@ TO \xFF\n"
 		"gomark		^Q 0 TO 9\n"
 		"gomark		^[ q 0 TO 9\n"
 		"groww		^K G\n"
@@ -379,6 +412,7 @@ const unsigned char * const builtins[]=
 		"save		^K S\n"
 		"save		^K ^S\n"
 		"save		^K s\n"
+		"scratch,\"(S) \"	^K -\n"
 		"setmark		^K 0 TO 9\n"
 		"shrinkw		^K T\n"
 		"shrinkw		^K ^T\n"
@@ -386,9 +420,9 @@ const unsigned char * const builtins[]=
 		"splitw		^K O\n"
 		"splitw		^K ^O\n"
 		"splitw		^K o\n"
-		"splitw,tag	^K ;\n"
 		"stat		^Q ?\n"
 		"stat		^[ q ?\n"
+		"tag		^K ;\n"
 		"tomarkb		^Q B\n"
 		"tomarkb		^Q ^B\n"
 		"tomarkb		^Q b\n"
@@ -479,10 +513,10 @@ const unsigned char * const builtins[]=
 		":inherit windows\n"
 		"\n"
 		":querya\n"
-		"type		^@ TO \n"
+		"type		^@ TO \xFF\n"
 		"\n"
 		":querysr\n"
-		"type		^@ TO \n"
+		"type		^@ TO \xFF\n"
 ,	NULL
-,	"@(#) $MirOS: contrib/code/jupp/builtins.c,v 1.5 2011/10/04 21:25:21 tg Exp $"
+,	"@(#) $MirOS: contrib/code/jupp/builtins.c,v 1.16 2013/01/05 22:16:29 tg Exp $"
 };
