@@ -1,4 +1,4 @@
-/* $MirOS: contrib/code/jupp/cmd.c,v 1.7 2010/01/03 17:54:07 tg Exp $ */
+/* $MirOS: contrib/code/jupp/cmd.c,v 1.11 2012/12/30 21:45:13 tg Exp $ */
 /*
  *	Command execution
  *	Copyright
@@ -9,6 +9,9 @@
 #include "config.h"
 #include "types.h"
 
+#ifdef HAVE_STDLIB_H
+#include <stdlib.h>
+#endif
 #include <string.h>
 
 #include "b.h"
@@ -16,6 +19,7 @@
 #include "cmd.h"
 #include "hash.h"
 #include "help.h"
+#include "kbd.h"
 #include "macro.h"
 #include "main.h"
 #include "menu.h"
@@ -55,6 +59,34 @@ int ubeep(BW *bw, int k)
 	return 0;
 }
 
+extern char main_context[];
+static int do_keymap(BW *bw, unsigned char *s, void *object, int *notify)
+{
+	KMAP *new_kmap;
+
+	if (notify)
+		*notify = 1;
+	if (!s || !*s || !(new_kmap = kmap_getcontext(s, 0)))
+		return (-1);
+	if (bw->o.context != (unsigned char *)main_context)
+		free(bw->o.context);
+	bw->o.context = strcmp((char *)s, main_context) ?
+	    (unsigned char *)strdup((char *)s) : (unsigned char *)main_context;
+	rmkbd(bw->parent->kbd);
+	bw->parent->kbd = mkkbd(new_kmap);
+	joe_snprintf_1((char *)msgbuf, JOE_MSGBUFSIZE, "New keymap: %s", s);
+	msgnw(bw->parent, msgbuf);
+	return (0);
+}
+static int ukeymap(BW *bw)
+{
+	if (wmkpw(bw->parent, US "Name of keymap to switch to: ", NULL,
+	    do_keymap, NULL, NULL, utypebw, NULL, NULL, locale_map)) {
+		return (0);
+	}
+	return (-1);
+}
+
 CMD cmds[] = {
 	{US "abort", TYPETW + TYPEPW + TYPEMENU + TYPEQW, uabort, NULL, 0, NULL},
 	{US "abortbuf", TYPETW, uabortbuf, NULL, 0, NULL},
@@ -99,7 +131,7 @@ CMD cmds[] = {
 	{US "dnslide", TYPETW + TYPEPW + TYPEMENU + TYPEQW + EMOVE, udnslide, NULL, 1, US "upslide"},
 	{US "drop", TYPETW + TYPEPW, udrop, NULL, 0, NULL},
 	{US "dupw", TYPETW, uduptw, NULL, 0, NULL},
-	{US "edit", TYPETW + TYPEPW, uedit, NULL, 0, NULL},
+	{US "edit", TYPETW, uedit, NULL, 0, NULL},
 	{US "eof", TYPETW + TYPEPW + EFIXXCOL + EMOVE, u_goto_eof, NULL, 0, NULL},
 	{US "eofmenu", TYPEMENU, umeof, NULL, 0, NULL},
 	{US "eol", TYPETW + TYPEPW + EFIXXCOL, u_goto_eol, NULL, 0, NULL},
@@ -121,11 +153,13 @@ CMD cmds[] = {
 	{US "killjoe", TYPETW + TYPEPW + TYPEMENU + TYPEQW, ukilljoe, NULL, 0, NULL},
 	{US "killproc", TYPETW + TYPEPW, ukillpid, NULL, 0, NULL},
 	{US "help", TYPETW + TYPEPW + TYPEQW, u_help, NULL, 0, NULL},
+	{US "helpcard", TYPETW + TYPEPW + TYPEQW, u_helpcard, NULL, 0, NULL},
 	{US "home", TYPETW + TYPEPW + EFIXXCOL, uhome, NULL, 0, NULL},
 	{US "hnext", TYPETW + TYPEPW + TYPEQW, u_help_next, NULL, 0, NULL},
 	{US "hprev", TYPETW + TYPEPW + TYPEQW, u_help_prev, NULL, 0, NULL},
 	{US "insc", TYPETW + TYPEPW + EFIXXCOL + EMOD, uinsc, NULL, 1, US "delch"},
 	{US "insf", TYPETW + TYPEPW + EMOD, uinsf, NULL, 0, NULL}, 
+	{US "keymap", TYPETW, ukeymap, NULL, 0, NULL},
 	{US "lindent", TYPETW + TYPEPW + EFIXXCOL + EMOD + EBLOCK, ulindent, NULL, 1, US "rindent"},
 	{US "line", TYPETW + TYPEPW, uline, NULL, 0, NULL},
 	{US "lose", TYPETW + TYPEPW, ulose, NULL, 0, NULL}, 
@@ -137,6 +171,8 @@ CMD cmds[] = {
 	{US "markk", TYPETW + TYPEPW, umarkk, NULL, 0, NULL},
 	{US "markl", TYPETW + TYPEPW, umarkl, NULL, 0, NULL},
 	{US "math", TYPETW + TYPEPW, umath, NULL, 0, NULL},
+	{US "mathins", TYPETW + TYPEPW, umathins, NULL, 0, NULL},
+	{US "mathres", TYPETW + TYPEPW, umathres, NULL, 0, NULL},
 	{US "mode", TYPETW + TYPEPW + TYPEQW, umode, NULL, 0, NULL},
 	{US "msg", TYPETW + TYPEPW + TYPEQW + TYPEMENU, umsg, NULL, 0, NULL},
 	{US "nbuf", TYPETW, unbuf, NULL, 1, US "pbuf"},
@@ -178,7 +214,7 @@ CMD cmds[] = {
 	{US "rtarw", TYPETW + TYPEPW /* + EFIXXCOL */, u_goto_right, NULL, 1, US "ltarw"}, /* EFIX removed for picture mode */
 	{US "rtarwmenu", TYPEMENU, umrtarw, NULL, 1, US "ltarwmenu"},
 	{US "rtn", TYPETW + TYPEPW + TYPEMENU + TYPEQW + EMOD, urtn, NULL, 1, NULL},
-	{US "save", TYPETW + TYPEPW, usave, NULL, 0, NULL},
+	{US "save", TYPETW, usave, NULL, 0, NULL},
 	{US "scratch", TYPETW + TYPEPW, uscratch, NULL, 0, NULL},
 	{US "select", TYPETW + TYPEPW, uselect, NULL, 0, NULL},
 	{US "setmark", TYPETW + TYPEPW, usetmark, NULL, 0, NULL},
@@ -391,3 +427,34 @@ int uexecmd(BW *bw)
 		return -1;
 	}
 }
+
+/*
+ * Show help screen at a specific card
+ */
+static int do_helpcard(BASE *base, unsigned char *s, void *object, int *notify)
+{
+	struct help *new_help;
+
+	if (notify)
+		*notify = 1;
+	if (!s || !*s) {
+		while (help_actual->prev != NULL)
+			/* find the first help entry */
+			help_actual = help_actual->prev;
+		help_off(base->parent->t);
+		return (0);
+	}
+	if ((new_help = find_context_help(s)) != NULL) {
+		help_actual = new_help;
+		return (help_on(base->parent->t));
+	}
+	return (-1);
+}
+int u_helpcard(BASE *base)
+{
+	if (wmkpw(base->parent, US "Name of help card to show: ", NULL,
+	    do_helpcard, NULL, NULL, utypebw, NULL, NULL, locale_map)) {
+		return (0);
+	}
+	return (-1);
+}
