@@ -1,6 +1,6 @@
-/* $MirOS: contrib/code/jupp/main.c,v 1.17 2012/06/07 23:38:17 tg Exp $ */
+/* $MirOS: contrib/code/jupp/main.c,v 1.20 2013/01/05 22:16:31 tg Exp $ */
 
-#define JUPP_IS_COPYRIGHT_C_BY "2012 mirabilos"
+#define JUPP_IS_COPYRIGHT_C_BY "2013 mirabilos"
 
 /*-
  * Copyright (c) 2004ff. Thorsten Glaser
@@ -67,6 +67,8 @@ unsigned char *exmsg = NULL;		/* Message to display when exiting the editor */
 
 SCREEN *maint;			/* Main edit screen */
 
+const char null[] = "";
+
 /* Make windows follow cursor */
 
 void dofollows(void)
@@ -399,7 +401,7 @@ int main(int argc, char **argv, char **envp)
 						/* update keymap */
 						rmkbd(bw->parent->kbd);
 						bw->parent->kbd =
-						    mkkbd(kmap_getcontext(bw->o.context));
+						    mkkbd(kmap_getcontext(bw->o.context, 1));
 					}
 				}
 				bw->b->o = bw->o;
