@@ -1,4 +1,4 @@
-/* $MirOS: contrib/code/jupp/scrn.c,v 1.8 2011/10/04 20:05:29 tg Exp $ */
+/* $MirOS: contrib/code/jupp/scrn.c,v 1.11 2012/12/30 21:45:17 tg Exp $ */
 /*
  *	Device independant TTY interface for JOE
  *	Copyright
@@ -27,6 +27,7 @@ int skiptop = 0;
 int lines = 0;
 int columns = 0;
 int notite = 0;
+int pastetite = 0;
 int usetabs = 0;
 int assume_color = 0;
 
@@ -477,17 +478,41 @@ SCRN *nopen(CAP *cap)
 	t->xn = getflag(t->cap,US "xn");
 	t->am = getflag(t->cap,US "am");
 
-	if (notite)
-		t->ti = 0;
-	else
-		t->ti = jgetstr(t->cap,US "ti");
 	t->cl = jgetstr(t->cap,US "cl");
 	t->cd = jgetstr(t->cap,US "cd");
 
-	if (notite)
-		t->te = 0;
-	else
+	if (notite) {
+		t->ti = NULL;
+		t->te = NULL;
+	} else {
+		t->ti = jgetstr(t->cap,US "ti");
 		t->te = jgetstr(t->cap,US "te");
+	}
+	if (pastetite && t->cap->paste_on && t->cap->paste_off) {
+		if (notite) {
+			t->ti = (void *)strdup(t->cap->paste_on);
+			t->te = (void *)strdup(t->cap->paste_off);
+		} else {
+			size_t n1, n2;
+			char *cp;
+
+			n1 = t->ti ? strlen(t->ti) : 0;
+			n2 = strlen(t->cap->paste_on);
+			cp = joe_malloc(n1 + n2 + 1);
+			if (t->ti)
+				memcpy(cp, t->ti, n1);
+			memcpy(cp + n1, t->cap->paste_on, n2 + 1);
+			t->ti = cp;
+
+			n1 = t->te ? strlen(t->te) : 0;
+			n2 = strlen(t->cap->paste_off);
+			cp = joe_malloc(n1 + n2 + 1);
+			memcpy(cp, t->cap->paste_off, n2 + 1);
+			if (t->te)
+				memcpy(cp + n2, t->te, n1 + 1);
+			t->te = cp;
+		}
+	}
 
 	t->ut = getflag(t->cap,US "ut");
 	t->Sb = jgetstr(t->cap,US "AB");
@@ -1023,7 +1048,7 @@ static void cposs(register SCRN *t, register int x, register int y)
 	if (t->cho + t->ccv < bestcost) {
 		cost = tcost(t->cap, t->ho, 1, 0, 0, 0, 0) + tcost(t->cap, t->cv, 1, y, 0, 0, 0) + relcost(t, x, y, 0, y);
 		if (cost < bestcost) {
-			bestcost = cost;
+			/* dead store: bestcost = cost; */
 			bestway = 12;
 		}
 	}
@@ -1897,7 +1922,7 @@ int txtwidth(unsigned char *s,int len)
 
 /* Generate text with formatting escape sequences */
 
-void genfmt(SCRN *t, int x, int y, int ofst, unsigned char *s, int flg)
+void genfmt(SCRN *t, int x, int y, int ofst, const unsigned char *s, int flg)
 {
 	int *scrn = t->scrn + y * t->co + x;
 	int *attr = t->attr + y * t->co + x;
@@ -1987,7 +2012,7 @@ void genfmt(SCRN *t, int x, int y, int ofst, unsigned char *s, int flg)
 
 /* Determine column width of string with format codes */
 
-int fmtlen(unsigned char *s)
+int fmtlen(const unsigned char *s)
 {
 	int col = 0;
 	struct utf8_sm sm;
