@@ -1,4 +1,4 @@
-/* $MirOS: contrib/code/jupp/types.h,v 1.8 2012/06/08 16:55:28 tg Exp $ */
+/* $MirOS: contrib/code/jupp/types.h,v 1.11 2012/12/30 21:45:17 tg Exp $ */
 
 #ifndef _JOE_TYPES_H
 #define _JOE_TYPES_H
@@ -12,6 +12,12 @@
 #include <sys/types.h>				/* we need pid_t */
 #endif
 
+#if defined(DEBUG) || defined(__COVERITY__)
+#define mkssert(e)	((e) ? (void)0 : exit(255))
+#else
+#define mkssert(e)	((void)0)
+#endif
+
 #define LINK(type) struct { type *next; type *prev; }
 
 #define KEYS		256
@@ -288,8 +294,8 @@ struct window {
 	} object;
 #endif
 
-	unsigned char	*msgt;		/* Message at top of window */
-	unsigned char	*msgb;		/* Message at bottom of window */
+	const unsigned char *msgt;	/* Message at top of window */
+	const unsigned char *msgb;	/* Message at bottom of window */
 	unsigned char	*huh;		/* Name of window for context sensitive hlp */
 	int	*notify;	/* Address of kill notification flag */
 };
@@ -488,6 +494,8 @@ struct cap {
 	void	*outptr;	/* First arg passed to output routine.  Second
 				   arg is character to write */
 	int	dopadding;	/* Set if pad characters should be used */
+	const char *paste_on;	/* Enable bracketed paste mode */
+	const char *paste_off;	/* Disable bracketed paste mode */
 };
 
 
