@@ -1,4 +1,4 @@
-/* $MirOS: contrib/code/jupp/ublock.c,v 1.6 2011/07/16 21:57:58 tg Exp $ */
+/* $MirOS: contrib/code/jupp/ublock.c,v 1.8 2012/12/30 18:43:40 tg Exp $ */
 /*
  * 	Highlighted block functions
  *	Copyright
@@ -29,6 +29,8 @@
 #include "uedit.h"
 #include "utils.h"
 #include "vs.h"
+#include "poshist.h"
+#include "ushell.h"
 #include "utf8.h"
 #include "charmap.h"
 #include "w.h"
@@ -916,8 +918,15 @@ int doinsf(BW *bw, unsigned char *s, void *object, int *notify)
 		if (error) {
 			msgnw(bw->parent, msgs[-error]), brm(tmp);
 			ret = -1;
-		} else
+		} else {
+			P *pafter;
+
+			pafter = pdup(bw->cursor);
+			pgetc(pafter);
 			binsb(bw->cursor, tmp);
+			prgetc(pafter);
+			aftermove(bw->parent, pafter);
+		}
 		vsrm(s);
 		bw->cursor->xcol = piscol(bw->cursor);
 		return ret;
@@ -959,6 +968,7 @@ static int dofilt(BW *bw, unsigned char *s, void *object, int *notify)
 	npartial(bw->parent->t->t);
 	ttclsn();
 	if (!fork()) {
+		const char *sh;
 #ifdef HAVE_PUTENV
 		unsigned char		*fname, *name;
 		unsigned	len;
@@ -984,7 +994,8 @@ static int dofilt(BW *bw, unsigned char *s, void *object, int *notify)
 		putenv((char *)fname);
 		vsrm(fname);
 #endif
-		execl("/bin/sh", "/bin/sh", "-c", s, NULL);
+		sh = getushell();
+		execl(sh, sh, "-c", s, NULL);
 		_exit(0);
 	}
 	close(fr[1]);
