@@ -1,4 +1,4 @@
-/* $MirOS: contrib/code/jupp/ufile.c,v 1.4 2010/04/08 15:31:05 tg Exp $ */
+/* $MirOS: contrib/code/jupp/ufile.c,v 1.8 2012/12/20 21:39:28 tg Exp $ */
 /*
  * 	User file operations
  *	Copyright
@@ -155,9 +155,10 @@ int ushell(BW *bw)
 
 /* Copy a file */
 
-static int cp(unsigned char *from, unsigned char *to)
+static int
+cp(unsigned char *from, int g, unsigned char *tmpfn, unsigned char *to)
 {
-	int f, g, amnt;
+	int f, amnt;
 	struct stat sbuf;
 
 #ifdef HAVEUTIME
@@ -175,10 +176,9 @@ static int cp(unsigned char *from, unsigned char *to)
 	if (fstat(f, &sbuf) < 0) {
 		return -1;
 	}
-	g = creat((char *)to, sbuf.st_mode & ~(S_ISUID | S_ISGID));
-	if (g < 0) {
+	if (fchmod(g, sbuf.st_mode & 0777)) {
 		close(f);
-		return -1;
+		return (-1);
 	}
 	while ((amnt = read(f, stdbuf, stdsiz)) > 0) {
 		if (amnt != joe_write(g, stdbuf, amnt)) {
@@ -186,11 +186,20 @@ static int cp(unsigned char *from, unsigned char *to)
 		}
 	}
 	close(f);
-	close(g);
 	if (amnt) {
 		return -1;
 	}
 
+	if (tmpfn && rename(tmpfn, to)) {
+		return (-1);
+	}
+	/*
+	 * Do not return !0 from here on.
+	 * Below are only operations that run when the copy
+	 * process finished successfully.
+	 */
+	close(g);
+
 #ifdef HAVEUTIME
 #ifdef NeXT
 	utbuf[0] = (time_t) sbuf.st_atime;
@@ -203,7 +212,7 @@ static int cp(unsigned char *from, unsigned char *to)
 #endif
 
 #ifdef WITH_SELINUX
-	copy_security_context(from,to);
+	copy_security_context(from, to);
 #endif
 
 	return 0;
@@ -214,59 +223,63 @@ static int cp(unsigned char *from, unsigned char *to)
  * Returns 1 for error
  */
 
-static int backup(BW *bw)
+static int
+backup(BW *bw)
 {
-	if (!bw->b->backup && !nobackups && bw->b->name && bw->b->name[0]) {
-		unsigned char tmp[1024];
-		unsigned char name[1024];
+	unsigned char tmp[1024 + 12];
+	unsigned char name[1024];
+	unsigned char *simple_backup_suffix;
+	int fd;
 
-#ifdef __MSDOS__
-		int x;
+	if (bw->b->backup || nobackups || !(bw->b->name) || !(bw->b->name[0]))
+		return (0);
 
-		if (backpath) {
-			joe_snprintf_2(name, sizeof(name), "%s/%s", backpath, namepart(tmp, bw->b->name));
-		} else {
-			joe_snprintf_1(name, sizeof(name), "%s", bw->b->name);
-		}
+	/* Create backup file name */
+	simple_backup_suffix = (unsigned char *)getenv("SIMPLE_BACKUP_SUFFIX");
 
-		for (x = strlen(name); name[--x] != '.';) {
-			if (name[x] == '\\' || (name[x] == ':' && x == 1) || x == 0) {
-				x = strlen(name);
-				break;
-			}
-		}
-
-		strcpy(name + x, ".bak");
-
-#else
-
-		/* Create backup file name */
-		unsigned char *simple_backup_suffix = (unsigned char *)getenv("SIMPLE_BACKUP_SUFFIX");
-		
-		if (simple_backup_suffix == NULL) {
-			simple_backup_suffix = US "~";
-		}
-		if (backpath) {
-			joe_snprintf_3((char *)name, sizeof(name), "%s/%s%s", backpath, namepart(tmp, bw->b->name), simple_backup_suffix);
-		} else {
-			joe_snprintf_2((char *)name, sizeof(name), "%s%s", bw->b->name, simple_backup_suffix);
-		}
-		
-		/* Attempt to delete backup file first */
-		unlink((char *)name);
+	if (simple_backup_suffix == NULL) {
+		simple_backup_suffix = US "~";
+	}
+	if (backpath) {
+		joe_snprintf_3((char *)name, sizeof(name), "%s/%s%s", backpath, namepart(tmp, bw->b->name), simple_backup_suffix);
+	} else {
+		joe_snprintf_2((char *)name, sizeof(name), "%s%s", bw->b->name, simple_backup_suffix);
+	}
 
+#ifdef HAVE_MKSTEMP
+	/* Securely generate a backup file temporary file */
+	joe_snprintf_1((char *)tmp, sizeof(tmp), "%s.XXXXXXXXXX", name);
+	if ((fd = mkstemp((char *)tmp)) < 0) {
+		return (1);
+	}
 #endif
 
-		/* Copy original file to backup file */
-		if (cp(bw->b->name, name)) {
-			return 1;
-		} else {
-			bw->b->backup = 1;
-			return 0;
-		}
-	} else {
-		return 0;
+	/* Attempt to delete backup file first */
+	unlink((char *)name);
+
+#ifdef HAVE_MKSTEMP
+	/* Copy original file to backup file securely */
+	if (cp(bw->b->name, fd, tmp, name)) {
+		close(fd);
+		unlink((char *)tmp);
+		return (1);
 	}
+#else
+	/* Yeowch! */
+	if ((fd = creat((char *)name, 0600)) < 0) {
+		return (1);
+	}
+#warning "TOCTOU temp file race here! Consider getting mkstemp!"
+
+	/* Copy original file to backup file */
+	if (cp(bw->b->name, fd, NULL, name)) {
+		close(fd);
+		unlink((char *)name);
+		return (1);
+	}
+#endif
+	bw->b->backup = 1;
+	return (0);
 }
 
 /* Write file */
@@ -468,7 +481,7 @@ static int dosave1(BW *bw, unsigned char *s, struct savereq *req, int *notify)
 int usave(BW *bw)
 {
 	BW *pbw;
-	
+
 	pbw = wmkpw(bw->parent, US "Name of file to save (^C to abort): ", &filehist, dosave1, US "Names", NULL, cmplt, mksavereq(NULL,NULL,NULL,0), NULL, locale_map);
 
 	if (pbw && bw->b->name) {
@@ -501,7 +514,7 @@ int doedit1(BW *bw,int c,unsigned char *s,int *notify)
 
 		b = bfind_reload(s);
 		er = error;
-		if (bw->b->count == 1 && (bw->b->changed || bw->b->name)) {
+		if (bw->b->count >= 1 && (bw->b->changed || bw->b->name)) {
 			if (orphan) {
 				orphit(bw);
 			} else {
